<?php

namespace Alipay\Kernel;

use Alipay\Kernel\Contracts\AccessTokenInterface;
use Alipay\Kernel\Http\Response;
use Alipay\Kernel\Traits\HasHttpRequests;
use Alipay\Kernel\Traits\SingData;
use GuzzleHttp\MessageFormatter;
use GuzzleHttp\Middleware;
use HttpBase\Events\HttpResponseCreated;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Log\LogLevel;

class BaseClient
{
    use SingData;
    use HasHttpRequests {
        request as performRequest;
    }

    /**
     * @var \Alipay\Kernel\ServiceContainer
     */
    protected $app;
    /**
     * @var AccessTokenInterface
     */
    protected $accessToken;
    /**
     * @var string
     */
    protected $baseUri;
    /**
     * @var string
     */
    protected $method;

    /**
     * BaseClient constructor.
     * @param ServiceContainer $app
     */
    public function __construct(ServiceContainer $app)
    {
        $this->app = $app;
    }

    /**
     * GET request.
     *
     * @param string $url
     * @param array $query
     * @return \Psr\Http\Message\ResponseInterface|\Alipay\Kernel\Support\Collection|array|object|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function httpGet(string $url, array $query = [])
    {
        return $this->request($url, 'GET', ['query' => $query]);
    }

    /**
     * POST request.
     *
     * @param array $data
     * @return \Psr\Http\Message\ResponseInterface|\Alipay\Kernel\Support\Collection|array|object|string
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \HttpBase\Exceptions\InvalidConfigException
     */
    public function httpPost(array $data = [])
    {
        $sysParams = $this->buildParams($data);
        $url = $this->_toUrlEncodedRequestBody($sysParams);
        return $this->request($url, 'POST', ['form_params' => $data]);
    }

    /**
     * JSON request.
     * @param string $url
     * @param array $data
     * @param array $query
     * @return \Psr\Http\Message\ResponseInterface|\Alipay\Kernel\Support\Collection|array|object|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function httpPostJson(string $url, array $data = [], array $query = [])
    {
        return $this->request($url, 'POST', ['query' => $query, 'json' => $data]);
    }

    /**
     * Upload file.
     *
     * @param array $data
     * @param null $body
     * @return Support\Collection|array|object|ResponseInterface|string
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \HttpBase\Exceptions\InvalidConfigException
     */
    public function httpUpload(array $data, $body = null)
    {
        $sysParams = $this->buildParams($data);
        $url = $this->_toUrlEncodedRequestBody($sysParams);
        return $this->request($url, 'POST', ['body' => $body, 'headers' => [
            "content-type" => "multipart/form-data;charset=utf-8;boundary=" . $this->_getRandomBoundary()
        ]]);
    }

    /**
     * @return AccessTokenInterface
     */
    public function getAccessToken(): AccessTokenInterface
    {
        return $this->accessToken;
    }

    /**
     * @param \Alipay\Kernel\Contracts\AccessTokenInterface $accessToken
     * @return $this
     */
    public function setAccessToken(AccessTokenInterface $accessToken)
    {
        $this->accessToken = $accessToken;
        return $this;
    }

    /**
     * @param string $url
     * @param string $method
     * @param array $options
     * @param bool $returnRaw
     * @return \Psr\Http\Message\ResponseInterface|\Alipay\Kernel\Support\Collection|array|object|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function request(string $url, string $method = 'GET', array $options = [], $returnRaw = false)
    {
        if (empty($this->middlewares)) {
            $this->registerHttpMiddlewares();
        }
        $response = $this->performRequest($url, $method, $options);
        $this->app->events->dispatch(new HttpResponseCreated($response));
        return $returnRaw ? $response : $this->castResponseToType($response, $this->app->config->get('response_type'));
    }

    /**
     * @param string $url
     * @param string $method
     * @param array $options
     * @return \Alipay\Kernel\Http\Response
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function requestRaw(string $url, string $method = 'GET', array $options = [])
    {
        return Response::buildFromPsrResponse($this->request($url, $method, $options, true), [
            'method' => $this->method,
            'sign_type' => $this->app->config->get('sign_type'),
            'alipay_public_key' => $this->app->config->get('alipay_public_key'),
        ]);
    }

    /**
     * Register Guzzle middlewares.
     */
    protected function registerHttpMiddlewares()
    {
        // retry
//        $this->pushMiddleware($this->retryMiddleware(), 'retry');
        // log
//        $this->pushMiddleware($this->accessTokenMiddleware(), 'access_token');

        $this->pushMiddleware($this->logMiddleware(), 'log');
    }

    /**
     * Attache access token to request query.
     *
     * @return \Closure
     */
    protected function accessTokenMiddleware()
    {
        return function (callable $handler) {
            return function (RequestInterface $request, array $options) use ($handler) {
                if ($this->accessToken) {
                    $request = $this->accessToken->applyToRequest($request, $options);
                }

                return $handler($request, $options);
            };
        };
    }

    /**
     * Log the request.
     *
     * @return \Closure
     */
    protected function logMiddleware()
    {
        $formatter = new MessageFormatter($this->app['config']['http.log_template'] ?? MessageFormatter::DEBUG);

        return Middleware::log($this->app['logger'], $formatter, LogLevel::DEBUG);
    }

    /**
     * Return retry middleware.
     *
     * @return \Closure
     */
    protected function retryMiddleware()
    {
        return Middleware::retry(function (
            $retries,
            RequestInterface $request,
            ResponseInterface $response = null
        ) {
            // Limit the number of retries to 2
            if ($retries < $this->app->config->get('http.max_retries', 1) && $response && $body = $response->getBody()) {
                // Retry on server errors
                $response = json_decode($body, true);
                if (!empty($response['errcode']) && in_array(abs($response['errcode']), [40001, 40014, 42001], true)) {
                    $this->accessToken->refresh();
                    $this->app['logger']->debug('Retrying with refreshed access token.');
                    return true;
                }
            }
            return false;
        }, function () {
            return abs($this->app->config->get('http.retry_delay', 500));
        });
    }
}