<?php

namespace Alipay\Kernel\Http;

use HttpBase\Exceptions\BadRequestException;
use HttpBase\Exceptions\InvalidArgumentException;
use Alipay\Kernel\Support\Collection;
use Alipay\Kernel\Support\XML;
use Alipay\Kernel\Traits\SingData;
use GuzzleHttp\Psr7\Response as GuzzleResponse;
use Psr\Http\Message\ResponseInterface;

/**
 * Class Response.
 *
 * @author overtrue <i@overtrue.me>
 */
class Response extends GuzzleResponse
{
    use SingData;

    protected $config;

    /**
     * @return string
     */
    public function getBodyContents()
    {
        $this->getBody()->rewind();
        $contents = $this->getBody()->getContents();
        $this->getBody()->rewind();

        return $contents;
    }

    /**
     * @param \Psr\Http\Message\ResponseInterface $response
     *
     * @return \Alipay\Kernel\Http\Response
     */
    public static function buildFromPsrResponse(ResponseInterface $response, $config)
    {
        $res = new static(
            $response->getStatusCode(),
            $response->getHeaders(),
            $response->getBody(),
            $response->getProtocolVersion(),
            $response->getReasonPhrase()
        );
        $res->config = $config;
        return $res;
    }

    /**
     * @return false|string
     * @throws BadRequestException
     * @throws InvalidArgumentException
     * @throws \HttpBase\Exceptions\InvalidConfigException
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * @return array|\SimpleXMLElement
     * @throws BadRequestException
     * @throws InvalidArgumentException
     * @throws \HttpBase\Exceptions\InvalidConfigException
     */
    public function toArray()
    {
        $content = $this->removeControlCharacters($this->getBodyContents());
        if (false !== stripos($this->getHeaderLine('Content-Type'), 'xml') || 0 === stripos($content, '<xml')) {
            return XML::parse($content);
        }
        $array = json_decode($content, true, 512, JSON_BIGINT_AS_STRING);
        if (JSON_ERROR_NONE === json_last_error()) {
            $method = str_replace(".", "_", $this->config['method']) . '_response';
            $content = $array[$method];
            if (!isset($array['sign']) || (isset($content['code']) && $content['code'] != '10000')) {
                throw new BadRequestException($content['msg'] . ' ' . $content['sub_code'], $content['code'] ?? 10000);
            }
            if (!$this->verify($content, $array['sign'], $this->config['alipay_public_key'], $this->config['sign_type'])) {
                throw new InvalidArgumentException('签名失败', 10000);
            }
            return (array)$content;
        }

        return [];
    }

    /**
     * @return Collection
     * @throws BadRequestException
     * @throws InvalidArgumentException
     * @throws \HttpBase\Exceptions\InvalidConfigException
     */
    public function toCollection()
    {
        return new Collection($this->toArray());
    }

    /**
     * @return mixed
     * @throws BadRequestException
     * @throws InvalidArgumentException
     * @throws \HttpBase\Exceptions\InvalidConfigException
     */
    public function toObject()
    {
        return json_decode($this->toJson());
    }

    /**
     * @return bool|string
     */
    public function __toString()
    {
        return $this->getBodyContents();
    }

    /**
     * @param string $content
     *
     * @return string
     */
    protected function removeControlCharacters(string $content)
    {
        return \preg_replace('/[\x00-\x1F\x80-\x9F]/u', '', $content);
    }
}