<?php

namespace Alipay\OpenPlatform\Authorizer\MiniProgram\Version;

use Alipay\Kernel\BaseClient;

class Client extends BaseClient
{
    /**
     * 小程序版本列表
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function list()
    {
        $params = [
            'biz_content' => null
        ];
        $this->method = 'alipay.open.mini.version.list.query';
        return $this->httpPost($params);
    }

    /**
     * 小程序版本详情
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function query(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.detail.query';
        return $this->httpPost($params);
    }

    /**
     * 上传构建小程序
     * @param string $template_id
     * @param string $app_version
     * @param array $payload
     * @param string $bundle_id
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function upload(string $template_id, string $app_version, array $payload, string $bundle_id = 'com.alipay.alipaywallet')
    {
        $biz_content = array_merge([
            'template_id' => $template_id,
            'app_version' => $app_version,
            'bundle_id' => $bundle_id,
        ], $payload);
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.upload';
        return $this->httpPost($params);
    }

    /**
     * 提交审核
     * @param string $appVersion
     * @param string $versionDesc
     * @param string $regionType
     * @param array $serviceRegionInfo
     * @param array $params
     * @param array $files
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function audit(string $appVersion, string $versionDesc, string $regionType, array $serviceRegionInfo = [], array $params = [], array $files = [])
    {
        $payload = [
            'app_version' => $appVersion,
            'version_desc' => $versionDesc,
            'region_type' => $regionType
        ];
        if ($regionType == 'LOCATION') {
            $payload['service_region_info'] = $serviceRegionInfo;
        }
        $biz_content = array_merge($payload, $params);
        $this->method = 'alipay.open.mini.version.audit.apply';
        $params = $biz_content;
        $boundary = $this->_getRandomBoundary();
        return $this->httpUpload($params, $this->_toMultipartRequestBody($biz_content, $files, $boundary));
    }

    /**
     * 上架小程序
     *
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function online(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId,
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.online';
        return $this->httpPost($params);
    }

    /**
     * 下架小程序
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function offline(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId,
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.offline';
        return $this->httpPost($params);
    }

    /**
     * 退回开发
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function cancel(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId,
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.audited.cancel';
        return $this->httpPost($params);
    }

    /**
     * 撤销审核
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function auditCancel(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId,
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.audit.cancel';
        return $this->httpPost($params);
    }

    /**
     * 回滚小程序
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function rollback(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId,
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.rollback';
        return $this->httpPost($params);
    }

    /**
     * 小程序删除版本
     *
     * @param string $appVersion
     * @param string $bundleId
     * @return \Alipay\Kernel\Support\Collection|array|object|\Psr\Http\Message\ResponseInterface|string
     * @throws \HttpBase\Exceptions\InvalidConfigException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function delete(string $appVersion, string $bundleId = 'com.alipay.alipaywallet')
    {
        $biz_content = [
            'app_version' => $appVersion,
            'bundle_id' => $bundleId,
        ];
        $params = [
            'biz_content' => json_encode($biz_content, JSON_UNESCAPED_UNICODE)
        ];
        $this->method = 'alipay.open.mini.version.delete';
        return $this->httpPost($params);
    }
}